#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>
#include <FL/Fl_Button.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Round_Button.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Hor_Nice_Slider.H>


Fl_Double_Window *main_win=(Fl_Double_Window *)0;
Fl_Button *test_bt=(Fl_Button *)0;
Fl_Round_Button *bt1=(Fl_Round_Button *)0;
Fl_Round_Button *bt2=(Fl_Round_Button *)0;
Fl_Round_Button *bt3=(Fl_Round_Button *)0;
Fl_Round_Button *bt0=(Fl_Round_Button *)0;
Fl_Hor_Nice_Slider *night_day=(Fl_Hor_Nice_Slider *)0;
Fl_Color l1, l2, l3;

unsigned col_buf [256];
unsigned ref_buf [256];

unsigned dimming = 0;

class alt_check : public Fl_Check_Button
{
public:
  alt_check(int X, int Y, int W, int H, const char *L) :
     Fl_Check_Button(X,Y,W,H,L) {};
private:
  void draw ()
  {
    Fl_Color old = Fl::get_color(FL_BACKGROUND2_COLOR); // save the default colour
    // force a green background, accounting for any dimming effect
    float dim_scale = 1.0 - ((float)dimming / 255.0);
    int dim_green = (int)(200.0 * dim_scale);
    Fl::set_color(FL_BACKGROUND2_COLOR, 0, dim_green, 0);
    Fl_Check_Button::draw(); // do the drawing
    Fl::set_color(FL_BACKGROUND2_COLOR, old); // put the default colour back
  }
};

alt_check *check1=(alt_check *)0;


static void cb_bt1(Fl_Round_Button*, void*) {
  Fl::set_color(FL_LIGHT1, 255, 0, 0);
  Fl::set_color(FL_LIGHT2, l2);
  Fl::set_color(FL_LIGHT3, l3);
  main_win->redraw();
}

static void cb_bt2(Fl_Round_Button*, void*) {
  Fl::set_color(FL_LIGHT1, l1);
  Fl::set_color(FL_LIGHT2, 255, 0, 0);
  Fl::set_color(FL_LIGHT3, l3);
  main_win->redraw();
}

static void cb_bt3(Fl_Round_Button*, void*) {
  Fl::set_color(FL_LIGHT1, l1);
  Fl::set_color(FL_LIGHT2, l2);
  Fl::set_color(FL_LIGHT3, 255, 0, 0);
  main_win->redraw();
}

static void cb_bt0(Fl_Round_Button*, void*) {
  Fl::set_color(FL_LIGHT1, l1);
  Fl::set_color(FL_LIGHT2, l2);
  Fl::set_color(FL_LIGHT3, l3);
  main_win->redraw();
}

static void set_dimming(void)
{
  int did_change = 0;
  unsigned dim_r = (dimming << 24) & 0xFF000000;
  unsigned dim_g = (dimming << 16) & 0x00FF0000;
  unsigned dim_b = (dimming <<  8) & 0x0000FF00;

  for (int idx = 0; idx < 256; ++idx)
  {
    unsigned uval = ref_buf[idx];
    if ((uval & 0xFF000000) >= dim_r)
    {
      uval = uval - dim_r;
    }
    if ((uval & 0x00FF0000) >= dim_g)
    {
      uval = uval - dim_g;
    }
    if ((uval & 0x0000FF00) >= dim_b)
    {
      uval = uval - dim_b;
    }
    if (col_buf[idx] != uval)
    {
      did_change = -1;
      col_buf[idx] = uval;
      Fl::set_color(idx, uval);
    }
  }
  if (did_change)
  {
    main_win->redraw();
  }
} // oscr_anim

static void cb_slider(Fl_Hor_Nice_Slider *w, void*) {
  dimming = (unsigned)w->value();
  if (dimming > 255) dimming = 255;
  set_dimming();
}

int main(int argc, char **argv) {
  { main_win = new Fl_Double_Window(492, 309, "Dimming Test");
    test_bt = new Fl_Button(30, 40, 115, 55, "Test");

    night_day = new Fl_Hor_Nice_Slider(200, 40, 200, 30, "Night - Day");
    night_day->bounds(175, 0);
    night_day->value(0);
    night_day->callback((Fl_Callback*)cb_slider);

    { Fl_Group* o = new Fl_Group(30, 110, 115, 140);
      o->box(FL_ENGRAVED_BOX);
      { bt1 = new Fl_Round_Button(39, 115, 55, 27, "1");
        bt1->type(102);
        bt1->down_box(FL_ROUND_DOWN_BOX);
        bt1->callback((Fl_Callback*)cb_bt1);
      } // Fl_Round_Button* bt1
      { bt2 = new Fl_Round_Button(39, 139, 55, 27, "2");
        bt2->type(102);
        bt2->down_box(FL_ROUND_DOWN_BOX);
        bt2->callback((Fl_Callback*)cb_bt2);
      } // Fl_Round_Button* bt2
      { bt3 = new Fl_Round_Button(39, 163, 55, 27, "3");
        bt3->type(102);
        bt3->down_box(FL_ROUND_DOWN_BOX);
        bt3->callback((Fl_Callback*)cb_bt3);
      } // Fl_Round_Button* bt3
      { bt0 = new Fl_Round_Button(39, 187, 55, 27, "0");
        bt0->type(102);
        bt0->down_box(FL_ROUND_DOWN_BOX);
        bt0->callback((Fl_Callback*)cb_bt0);
        bt0->value(1);
      } // Fl_Round_Button* bt0

      check1= new alt_check(39, 220, 55, 27, "C");
      check1->selection_color(FL_RED);

      o->end();
    } // Fl_Group* o
    main_win->end();
  } // Fl_Double_Window* main_win

  l1 = Fl::get_color(FL_LIGHT1);
  l2 = Fl::get_color(FL_LIGHT2);
  l3 = Fl::get_color(FL_LIGHT3);

  main_win->show(argc, argv);

  // copy the colour set
  for (int idx = 0; idx < 256; ++idx)
  {
    col_buf[idx] = Fl::get_color(idx);
    ref_buf[idx] = col_buf[idx];
  }

  return Fl::run();
}

// end of file //
